@@ -14,6 +14,8 @@ from subprocess import check_call, CalledProcessError
 import sys
 import importlib
 
+from glue._plugin_helpers import iter_plugin_entry_points
+
 
 class Dependency(object):
 
@@ -37,8 +39,13 @@ class Dependency(object):
         try:
             module = __import__(self.module)
             return module.__version__
-        except (ImportError, AttributeError):
+        except ImportError:
             return 'unknown version'
+        except AttributeError:
+            try:
+                return module.__VERSION__
+            except AttributeError:
+                return 'unknown version'
 
     def install(self):
         if self.installed:
@@ -70,7 +77,21 @@ PIP package name:
             status = 'FAILED (%s)' % self.info
         else:
             status = 'MISSING (%s)' % self.info
-        return "%20s:\t%s" % (self.module, status)
+        return "%20s:\t%s" % (self.package, status)
+
+
+class Python(Dependency):
+
+    def __init__(self):
+        self.package = 'Python'
+
+    @property
+    def installed(self):
+        return True
+
+    @property
+    def version(self):
+        return sys.version.split()[0]
 
 
 class QtDependency(Dependency):
@@ -86,17 +107,6 @@ class QtDependency(Dependency):
         return "%20s:\t%s" % (self.module, status)
 
 
-class PyQt4(QtDependency):
-
-    @property
-    def version(self):
-        try:
-            from PyQt4 import Qt
-            return "PyQt: {0} - Qt: {1}".format(Qt.PYQT_VERSION_STR, Qt.QT_VERSION_STR)
-        except (ImportError, AttributeError):
-            return 'unknown version'
-
-
 class PyQt5(QtDependency):
 
     @property
@@ -108,39 +118,45 @@ class PyQt5(QtDependency):
             return 'unknown version'
 
 
-class PySide(QtDependency):
+class PySide2(QtDependency):
 
     @property
     def version(self):
         try:
-            import PySide
-            from PySide import QtCore
-            return "PySide: {0} - Qt: {1}".format(PySide.__version__, QtCore.__version__)
+            import PySide2
+            from PySide2 import QtCore
+            return "PySide2: {0} - Qt: {1}".format(PySide2.__version__, QtCore.__version__)
         except (ImportError, AttributeError):
             return 'unknown version'
 
 
 # Add any dependencies here
 # Make sure to add new categories to the categories tuple
+
+python = (
+    Python(),
+)
+
 gui_framework = (
-    PyQt4('PyQt4', ''),
     PyQt5('PyQt5', ''),
-    PySide('PySide', '')
+    PySide2('PySide2', '')
 )
 
 required = (
-    Dependency('qtpy', 'Required'),
-    Dependency('setuptools', 'Required'),
-    Dependency('numpy', 'Required', min_version='1.4'),
-    Dependency('matplotlib', 'Required for plotting', min_version='1.1'),
-    Dependency(
-        'pandas', 'Adds support for Excel files and DataFrames', min_version='0.13.1'),
-    Dependency('astropy', 'Used for FITS I/O, table reading, and WCS Parsing'))
+    Dependency('qtpy', 'Required', min_version='1.2'),
+    Dependency('setuptools', 'Required', min_version='1.0'),
+    Dependency('numpy', 'Required', min_version='1.9'),
+    Dependency('bottleneck', 'Required', min_version='1.2'),
+    Dependency('matplotlib', 'Required for plotting', min_version='2.0'),
+    Dependency('pandas', 'Adds support for Excel files and DataFrames', min_version='0.14'),
+    Dependency('astropy', 'Used for FITS I/O, table reading, and WCS Parsing', min_version='1.3'),
+    Dependency('dill', 'Used when saving Glue sessions', min_version='0.2'),
+    Dependency('h5py', 'Used to support HDF5 files', min_version='2.4'),
+    Dependency('xlrd', 'Used to support Excel files', min_version='1.0'),
+    Dependency('mpl_scatter_density', 'Used to make fast scatter density plots', 'mpl-scatter-density', min_version='0.3'),
+)
 
 general = (
-    Dependency('dill', 'Used when saving Glue sessions'),
-    Dependency('h5py', 'Used to support HDF5 files'),
-    Dependency('xlrd', 'Used to support Excel files'),
     Dependency('scipy', 'Used for some image processing calculation'),
     Dependency('skimage',
                'Used to read popular image formats (jpeg, png, etc.)',
@@ -148,9 +164,9 @@ general = (
 
 
 ipython = (
-    Dependency('IPython', 'Needed for interactive IPython terminal'),
-    Dependency('ipykernel', 'Needed for interactive IPython terminal'),
+    Dependency('IPython', 'Needed for interactive IPython terminal', min_version='4'),
     Dependency('qtconsole', 'Needed for interactive IPython terminal'),
+    Dependency('ipykernel', 'Needed for interactive IPython terminal'),
     Dependency('traitlets', 'Needed for interactive IPython terminal'),
     Dependency('pygments', 'Needed for interactive IPython terminal'),
     Dependency('zmq', 'Needed for interactive IPython terminal', 'pyzmq'))
@@ -159,7 +175,6 @@ ipython = (
 astronomy = (
     Dependency('pyavm', 'Used to parse AVM metadata in image files', 'PyAVM'),
     Dependency('spectral_cube', 'Used to read in spectral cubes', 'spectral-cube'),
-    Dependency('ginga', 'Adds a ginga viewer to glue', 'ginga'),
     Dependency('astrodendro', 'Used to read in and represent dendrograms', 'astrodendro'))
 
 
@@ -171,15 +186,31 @@ export = (
     Dependency('plotly', 'Used to explort plots to Plot.ly'),
 )
 
-categories = (('gui framework', gui_framework),
+
+def plugins():
+    modules = []
+    dependencies = []
+    for entry_point in iter_plugin_entry_points():
+        module_name = entry_point.module_name.split('.')[0]
+        package = entry_point.dist.project_name
+        modules.append((module_name, package))
+    for module, package in sorted(set(modules)):
+        dependencies.append(Dependency(module, '', package=package))
+    return dependencies
+
+
+categories = (('python', python),
+              ('gui framework', gui_framework),
               ('required', required),
-              ('general', general),
+              ('plugins', plugins()),
               ('ipython terminal', ipython),
+              ('general', general),
               ('astronomy', astronomy),
               ('testing', testing),
               ('export', export))
 
-dependencies = dict((d.module, d) for c in categories for d in c[1])
+
+dependencies = dict((d.package, d) for c in categories for d in c[1])
 
 
 def get_status():
@@ -197,9 +228,9 @@ def get_status_as_odict():
     for category, deps in categories:
         for dep in deps:
             if dep.installed:
-                status[dep.module] = dep.version
+                status[dep.package] = dep.version
             else:
-                status[dep.module] = "Not installed"
+                status[dep.package] = "Not installed"
     return status
 
 
