@@ -1,3 +1,6 @@
+from __future__ import absolute_import, division, print_function
+
+
 class MatplotlibBackendSetter(object):
     """
     Import hook to make sure the proper Qt backend is set when importing
@@ -12,13 +15,16 @@ class MatplotlibBackendSetter(object):
             set_mpl_backend()
 
     def find_spec(self, name, import_path, target_module=None):
-        pass
+        if self.enabled and name.startswith('matplotlib'):
+            self.enabled = False
+            set_mpl_backend()
+
 
 def set_mpl_backend():
 
     try:
-        from qtpy import PYQT5
-    except:
+        from qtpy import PYQT5  # noqa
+    except Exception:
         # If Qt isn't available, we don't have to worry about
         # setting the backend
         return
@@ -28,10 +34,12 @@ def set_mpl_backend():
     # standardize mpl setup
     rcdefaults()
 
-    if PYQT5:
-        rcParams['backend'] = 'Qt5Agg'
-    else:
-        rcParams['backend'] = 'Qt4Agg'
+    rcParams['backend'] = 'Qt5Agg'
+
+    # disable key bindings in matplotlib
+    for setting in list(rcParams.keys()):
+        if setting.startswith('keymap'):
+            rcParams[setting] = ''
 
     # The following is a workaround for the fact that Matplotlib checks the
     # rcParams at import time, not at run-time. I have opened an issue with
