@@ -6,21 +6,27 @@ editing the data collection
 from __future__ import absolute_import, division, print_function
 
 import os
+import weakref
+
+from qtpy import QtCore, QtWidgets, QtGui
+from qtpy.QtCore import Qt
 
-from glue.external.qt.QtCore import Qt
-from glue.external.qt import QtGui, QtCore
 from glue.core.edit_subset_mode import AndMode, OrMode, XorMode, AndNotMode
-from glue.core.qt.data_collection_model import DataCollectionView
-from glue.config import single_subset_action
+from glue.config import layer_action
 from glue import core
 from glue.dialogs.link_editor.qt import LinkEditor
 from glue.icons.qt import get_icon
 from glue.app.qt.actions import action
-from glue.dialogs.custom_component.qt import CustomComponentWidget
+from glue.dialogs.component_arithmetic.qt import ArithmeticEditorWidget
+from glue.dialogs.component_manager.qt import ComponentManagerWidget
 from glue.dialogs.subset_facet.qt import SubsetFacet
 from glue.dialogs.data_wizard.qt import data_wizard
 from glue.utils import nonpartial
+from glue.utils.decorators import avoid_circular
 from glue.utils.qt import load_ui
+from glue.core.message import EditSubsetMessage
+from glue.core.hub import HubListener
+from glue.app.qt.metadata import MetadataDialog
 
 
 @core.decorators.singleton
@@ -30,7 +36,7 @@ class Clipboard(object):
         self.contents = None
 
 
-class LayerAction(QtGui.QAction):
+class LayerAction(QtWidgets.QAction):
     _title = ''
     _icon = None
     _tooltip = None
@@ -40,7 +46,7 @@ class LayerAction(QtGui.QAction):
 
     def __init__(self, layer_tree_widget):
         self._parent = layer_tree_widget.ui.layerTree
-        super(LayerAction, self).__init__(self._title.title(), self._parent)
+        super(LayerAction, self).__init__(self._title, self._parent)
         self._layer_tree = layer_tree_widget
         if self._icon:
             self.setIcon(get_icon(self._icon))
@@ -53,6 +59,7 @@ class LayerAction(QtGui.QAction):
             self.setShortcut(self._shortcut)
         self._parent.addAction(self)
         self._connect()
+        self.setIconVisibleInMenu(False)
 
     def _connect(self):
         self._parent.selection_changed.connect(nonpartial(self.update_enabled))
@@ -73,6 +80,12 @@ class LayerAction(QtGui.QAction):
     def single_selection(self):
         return len(self.selected_layers()) == 1
 
+    def single_selection_data(self):
+        layers = self.selected_layers()
+        if len(layers) != 1:
+            return False
+        return isinstance(layers[0], core.Data)
+
     def single_selection_subset(self):
         layers = self.selected_layers()
         if len(layers) != 1:
@@ -95,12 +108,12 @@ class LayerAction(QtGui.QAction):
 class PlotAction(LayerAction):
 
     """Visualize the selection. Requires GlueApplication"""
-    _title = "Plot Data"
-    _tooltip = "Make a plot of this selection"
+    _title = "Create new viewer"
+    _tooltip = "Create a new viewer from this object"
 
     def __init__(self, tree, app):
         super(PlotAction, self).__init__(tree)
-        self.app = app
+        self.app = weakref.ref(app)
 
     def _can_trigger(self):
         if not self.single_selection():
@@ -110,7 +123,9 @@ class PlotAction(LayerAction):
     def _do_action(self):
         assert self._can_trigger()
         data = self.selected_layers()[0].data
-        self.app.choose_new_data_viewer(data)
+        app = self.app()
+        if app is not None:
+            app.choose_new_data_viewer(data)
 
 
 class FacetAction(LayerAction):
@@ -126,17 +141,35 @@ class FacetAction(LayerAction):
         layers = self.selected_layers()
         try:
             default = layers[0].data
-        except (AttributeError, TypeError):
+        except (AttributeError, TypeError, IndexError):
             default = None
         SubsetFacet.facet(self._layer_tree.data_collection,
                           parent=self._layer_tree, default=default)
 
 
+class MetadataAction(LayerAction):
+
+    _title = "View metadata/header"
+    _tooltip = "View metadata/header"
+    _shortcut = QtGui.QKeySequence('Ctrl+I')
+
+    def _can_trigger(self):
+        return self.single_selection_data() or self.single_selection_subset()
+
+    def _do_action(self):
+        layers = self.selected_layers()
+        if isinstance(layers[0], core.Subset):
+            data = layers[0].data
+        else:
+            data = layers[0]
+        MetadataDialog(data).exec_()
+
+
 class NewAction(LayerAction):
     _title = "New Subset"
     _tooltip = "Create a new subset"
-    _icon = "glue_subset"
     _shortcut = QtGui.QKeySequence('Ctrl+Shift+N')
+    _icon = 'glue_subset'
 
     def _can_trigger(self):
         return len(self.data_collection) > 0
@@ -185,12 +218,10 @@ class LinkAction(LayerAction):
     _title = "Link Data"
     _tooltip = "Define links between data sets"
     _data_link_message = "Define links between data sets"
-    _icon = "glue_link"
+    _icon = 'glue_link'
 
     def __init__(self, *args, **kwargs):
         super(LinkAction, self).__init__(*args, **kwargs)
-        self._link_icon = get_icon(self._icon)
-        self._unlink_icon = get_icon('glue_unlink')
 
     def _can_trigger(self):
         return len(self.data_collection) > 0
@@ -212,17 +243,59 @@ class MaskifySubsetAction(LayerAction):
         s.subset_state = s.state_as_mask()
 
 
-class SaveAction(LayerAction):
-    _title = "Save subset"
-    _tooltip = "Save the mask for this subset to a file"
+class ExportDataAction(LayerAction):
+
+    _title = "Export data values"
+    _tooltip = "Save the data to a file"
+
+    def _can_trigger(self):
+        return self.single_selection_data()
+
+    def _do_action(self):
+        assert self._can_trigger()
+        data = self.selected_layers()[0]
+        from glue.core.data_exporters.qt.dialog import export_data
+        export_data(data)
+
+
+class ExportSubsetAction(ExportDataAction):
+
+    _title = "Export subset values"
+    _tooltip = "Save the data subset to a file"
 
     def _can_trigger(self):
         return self.single_selection_subset()
 
+
+class ImportSubsetMaskAction(LayerAction):
+
+    _title = "Import subset mask(s)"
+    _tooltip = "Import subset mask from a file"
+
+    def _can_trigger(self):
+        return self.single_selection_subset() or self.single_selection_data()
+
     def _do_action(self):
         assert self._can_trigger()
-        subset = self.selected_layers()[0]
-        save_subset(subset)
+        data = self.selected_layers()[0]
+        from glue.io.qt.subset_mask import QtSubsetMaskImporter
+        QtSubsetMaskImporter().run(data, self._layer_tree._data_collection)
+
+
+class ExportSubsetMaskAction(LayerAction):
+
+    _title = "Export subset mask(s)"
+    _tooltip = "Export subset mask to a file"
+
+    def _can_trigger(self):
+        return (len(self.data_collection.subset_groups) > 0 and
+                (self.single_selection_subset() or self.single_selection_data()))
+
+    def _do_action(self):
+        assert self._can_trigger()
+        data = self.selected_layers()[0]
+        from glue.io.qt.subset_mask import QtSubsetMaskExporter
+        QtSubsetMaskExporter().run(data)
 
 
 class CopyAction(LayerAction):
@@ -268,24 +341,24 @@ class PasteSpecialAction(PasteAction):
         self.setMenu(self.menu())
 
     def menu(self):
-        m = QtGui.QMenu()
+        m = QtWidgets.QMenu()
 
-        a = QtGui.QAction("Or", m)
+        a = QtWidgets.QAction("Or", m)
         a.setIcon(get_icon('glue_or'))
         a.triggered.connect(nonpartial(self._paste, OrMode))
         m.addAction(a)
 
-        a = QtGui.QAction("And", m)
+        a = QtWidgets.QAction("And", m)
         a.setIcon(get_icon('glue_and'))
         a.triggered.connect(nonpartial(self._paste, AndMode))
         m.addAction(a)
 
-        a = QtGui.QAction("XOR", m)
+        a = QtWidgets.QAction("XOR", m)
         a.setIcon(get_icon('glue_xor'))
         a.triggered.connect(nonpartial(self._paste, XorMode))
         m.addAction(a)
 
-        a = QtGui.QAction("Not", m)
+        a = QtWidgets.QAction("Not", m)
         a.setIcon(get_icon('glue_andnot'))
         a.triggered.connect(nonpartial(self._paste, AndNotMode))
         m.addAction(a)
@@ -303,8 +376,7 @@ class PasteSpecialAction(PasteAction):
 
 
 class Inverter(LayerAction):
-    _title = "Invert"
-    _icon = "glue_not"
+    _title = "Invert Subset"
     _tooltip = "Invert selected subset"
 
     def _can_trigger(self):
@@ -338,40 +410,58 @@ class MergeAction(LayerAction):
 
 
 class UserAction(LayerAction):
+    """
+    User-defined callback functions to expose.
 
-    def __init__(self, layer_tree_widget, callback, **kwargs):
-        self._title = kwargs.get('name', 'User Action')
-        self._tooltip = kwargs.get('tooltip', None)
-        self._icon = kwargs.get('icon', None)
+    Users register new actions via the :member:`glue.config.layer_action` member
+
+    Callback functions are passed the layer (or list of layers) and data
+    collection
+    """
+
+    def __init__(self, layer_tree_widget, callback=None, label='User Action',
+                 tooltip=None, icon=None, single=False, data=False,
+                 subset_group=False, subset=False):
+        self._title = label
+        self._tooltip = tooltip
+        self._icon = icon
+        self._single = single
+        self._data = data
+        self._subset_group = subset_group
+        self._subset = subset
         self._callback = callback
         super(UserAction, self).__init__(layer_tree_widget)
 
-
-class SingleSubsetUserAction(UserAction):
-
-    """
-    User-defined callback functions to expose
-    when single subsets are selected.
-
-    Users register new actions via the :member:`glue.config.single_subset_action`
-    member
-
-    Callback functions are passed the subset and data collection
-    """
-
     def _can_trigger(self):
-        return self.single_selection_subset()
+        if self._single:
+            if self.single_selection():
+                layer = self.selected_layers()[0]
+                if isinstance(layer, core.Data) and self._data:
+                    return True
+                elif isinstance(layer, core.SubsetGroup) and self._subset_group:
+                    return True
+                elif isinstance(layer, core.Subset) and self._subset:
+                    return True
+                else:
+                    return False
+            else:
+                return False
+        else:
+            return len(self.selected_layers()) > 0
 
     def _do_action(self):
-        subset, = self.selected_layers()
-        return self._callback(subset, self.data_collection)
+        if self._single:
+            subset = self.selected_layers()[0]
+            return self._callback(subset, self.data_collection)
+        else:
+            return self._callback(self.selected_layers(), self.data_collection)
 
 
 class LayerCommunicator(QtCore.QObject):
     layer_check_changed = QtCore.Signal(object, bool)
 
 
-class LayerTreeWidget(QtGui.QMainWindow):
+class LayerTreeWidget(QtWidgets.QMainWindow, HubListener):
 
     """The layertree widget provides a way to visualize the various
     data and subset layers in a Glue session.
@@ -381,14 +471,14 @@ class LayerTreeWidget(QtGui.QMainWindow):
     it isn't attached to a hub, interactions may not propagate properly.
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, session=None, parent=None):
 
         super(LayerTreeWidget, self).__init__(parent)
 
+        self.session = session
+
         self.ui = load_ui('layer_tree_widget.ui', None, directory=os.path.dirname(__file__))
         self.setCentralWidget(self.ui)
-        self.ui.layerAddButton.setIcon(get_icon('glue_open'))
-        self.ui.layerRemoveButton.setIcon(get_icon('glue_delete'))
 
         self._signals = LayerCommunicator()
         self._is_checkable = True
@@ -398,10 +488,10 @@ class LayerTreeWidget(QtGui.QMainWindow):
         self._actions = {}
 
         self._create_actions()
-        self._connect()
         self._data_collection = None
         self._hub = None
         self.ui.layerTree.setDragEnabled(True)
+        self.setMinimumSize(300, 0)
 
     @property
     def data_collection(self):
@@ -411,6 +501,7 @@ class LayerTreeWidget(QtGui.QMainWindow):
         self._data_collection = collection
         self._hub = collection.hub
         self.ui.layerTree.set_data_collection(collection)
+        self.bind_selection_to_edit_subset()
 
     def unregister(self, hub):
         """Unsubscribe from hub"""
@@ -438,45 +529,47 @@ class LayerTreeWidget(QtGui.QMainWindow):
         """ Return the list of actions attached to this widget """
         return self.ui.layerTree.actions()
 
-    def _connect(self):
-        """ Connect widget signals to methods """
-        self._actions['link'] = LinkAction(self)
-        self.ui.layerAddButton.clicked.connect(nonpartial(self._load_data))
-        self.ui.layerRemoveButton.clicked.connect(self._actions['delete'].trigger)
-        self.ui.linkButton.set_action(self._actions['link'])
-        self.ui.newSubsetButton.set_action(self._actions['new'], text=False)
-
-        rbut = self.ui.layerRemoveButton
-
-        def update_enabled():
-            return rbut.setEnabled(self._actions['delete'].isEnabled())
-        self.ui.layerTree.selection_changed.connect(update_enabled)
-
     def bind_selection_to_edit_subset(self):
-        self.ui.layerTree.selection_changed.connect(
-            self._update_editable_subset)
+        self.ui.layerTree.selection_changed.connect(self._update_edit_subset)
+        self._data_collection.hub.subscribe(self, EditSubsetMessage,
+                                            handler=self._update_subset_selection)
 
-    def _update_editable_subset(self):
-        """Update edit subsets to match current selection"""
-        layers = self.selected_layers()
-        layers.extend(s for l in layers
-                      if isinstance(l, core.SubsetGroup)
-                      for s in l.subsets)
+    @avoid_circular
+    def _update_subset_selection(self, message):
+        """
+        Update current selection to match edit subsets
+        """
+        self.ui.layerTree.set_selected_layers(message.subset)
 
-        for data in self.data_collection:
-            data.edit_subset = [s for s in data.subsets if s in layers]
+    @avoid_circular
+    def _update_edit_subset(self):
+        """
+        Update edit subsets to match current selection
+        """
+        mode = self.session.edit_subset_mode
+        mode.edit_subset = [s for s in self.selected_layers() if isinstance(s, core.SubsetGroup)]
 
     def _create_component(self):
-        CustomComponentWidget.create_component(self.data_collection)
+        dialog = ArithmeticEditorWidget(self.data_collection)
+        dialog.exec_()
+
+    def _manage_components(self):
+        dialog = ComponentManagerWidget(self.data_collection)
+        dialog.exec_()
 
     def _create_actions(self):
         tree = self.ui.layerTree
 
-        sep = QtGui.QAction("", tree)
+        sep = QtWidgets.QAction("", tree)
         sep.setSeparator(True)
         tree.addAction(sep)
 
-        self._actions['save'] = SaveAction(self)
+        # Actions relating to I/O
+        self._actions['save_data'] = ExportDataAction(self)
+        self._actions['save_subset'] = ExportSubsetAction(self)
+        self._actions['import_subset_mask'] = ImportSubsetMaskAction(self)
+        self._actions['export_subset_mask'] = ExportSubsetMaskAction(self)
+
         self._actions['copy'] = CopyAction(self)
         self._actions['paste'] = PasteAction(self)
         self._actions['paste_special'] = PasteSpecialAction(self)
@@ -485,26 +578,32 @@ class LayerTreeWidget(QtGui.QMainWindow):
         self._actions['clear'] = ClearAction(self)
         self._actions['delete'] = DeleteAction(self)
         self._actions['facet'] = FacetAction(self)
+        self._actions['metadata'] = MetadataAction(self)
         self._actions['merge'] = MergeAction(self)
         self._actions['maskify'] = MaskifySubsetAction(self)
+        self._actions['link'] = LinkAction(self)
 
-        # new component definer
-        separator = QtGui.QAction("sep", tree)
-        separator.setSeparator(True)
-        tree.addAction(separator)
+        sep = QtWidgets.QAction("", tree)
+        sep.setSeparator(True)
+        tree.addAction(sep)
 
-        a = action("Define new component", self,
-                 tip="Define a new component using python expressions")
+        a = action("Add/edit arithmetic attributes", self,
+                   tip="Add/edit attributes derived from existing ones")
         tree.addAction(a)
         a.triggered.connect(nonpartial(self._create_component))
         self._actions['new_component'] = a
 
-        # user-defined layer actions
-        for name, callback, tooltip, icon in single_subset_action:
-            self._actions[name] = SingleSubsetUserAction(self, callback,
-                                                         name=name,
-                                                         tooltip=tooltip,
-                                                         icon=icon)
+        a = action("Reorder/rename data attributes", self,
+                   tip="Reorder/rename data attributes")
+        tree.addAction(a)
+        a.triggered.connect(nonpartial(self._manage_components))
+        self._actions['manage_components'] = a
+
+        # Add user-defined layer actions. Note that _asdict is actually a public
+        # method, but just has an underscore to prevent conflict with
+        # namedtuple attributes.
+        for item in layer_action:
+            self._actions[item.label] = UserAction(self, **item._asdict())
 
         # right click pulls up menu
         tree.setContextMenuPolicy(Qt.ActionsContextMenu)
@@ -540,19 +639,10 @@ class LayerTreeWidget(QtGui.QMainWindow):
         return len(self.ui.layerTree)
 
 
-def save_subset(subset):
-    assert isinstance(subset, core.subset.Subset)
-    fname, fltr = QtGui.QFileDialog.getSaveFileName(caption="Select an output name",
-                                                    filter='FITS mask (*.fits);; Fits mask (*.fits)')
-    fname = str(fname)
-    if not fname:
-        return
-    subset.write_mask(fname)
-
 if __name__ == "__main__":
     from glue.core.data_collection import DataCollection
     collection = DataCollection()
-    from glue.external.qt import get_qapp
+    from glue.utils.qt import get_qapp
     app = get_qapp()
     widget = LayerTreeWidget()
     widget.setup(collection)
