@@ -1,17 +1,18 @@
 from __future__ import absolute_import, division, print_function
 
+import os
 import traceback
 from functools import wraps
 
+from glue.external.six import string_types
 from glue.core.session import Session
-from glue.core.edit_subset_mode import EditSubsetMode
 from glue.core.hub import HubListener
 from glue.core import Data, Subset
 from glue.core import command
 from glue.core.data_factories import load_data
 from glue.core.data_collection import DataCollection
 from glue.config import settings
-from glue.utils import as_list, PropertySetMixin
+from glue.utils import PropertySetMixin
 
 
 __all__ = ['Application', 'ViewerBase']
@@ -32,6 +33,16 @@ def catch_error(msg):
     return decorator
 
 
+def as_flat_data_list(data):
+    datasets = []
+    if isinstance(data, Data):
+        datasets.append(data)
+    else:
+        for d in data:
+            datasets.extend(as_flat_data_list(d))
+    return datasets
+
+
 class Application(HubListener):
 
     def __init__(self, data_collection=None, session=None):
@@ -44,10 +55,9 @@ class Application(HubListener):
             self._session = Session(data_collection=self._data,
                                     application=self)
 
-        EditSubsetMode().data_collection = self._data
         self._hub = self._session.hub
         self._cmds = self._session.command_stack
-        self._cmds.add_callback(lambda x: self._update_undo_redo_enabled())
+        self._cmds.add_callback(self._update_undo_redo_enabled)
 
         self._settings = {}
         for key, value, validator in settings:
@@ -81,7 +91,7 @@ class Application(HubListener):
         return c
 
     @catch_error("Failed to save session")
-    def save_session(self, path, include_data=False):
+    def save_session(self, path, include_data=False, absolute_paths=True):
         """ Save the data collection and hub to file.
 
         Can be restored via restore_session
@@ -89,9 +99,25 @@ class Application(HubListener):
         Note: Saving of client is not currently supported. Thus,
         restoring this session will lose all current viz windows
         """
+
         from glue.core.state import GlueSerializer
-        gs = GlueSerializer(self, include_data=include_data)
-        state = gs.dumps(indent=2)
+        gs = GlueSerializer(self,
+                            include_data=include_data,
+                            absolute_paths=absolute_paths)
+
+        # In case relative paths are needed in the session file, we do the
+        # serialization while setting the current directory to the directory
+        # in which the session file will be saved so that the relative paths
+        # are relative to the session file, not the current working directory.
+        start_dir = os.path.abspath('.')
+        session_dir = os.path.dirname(path) or '.'
+
+        try:
+            os.chdir(session_dir)
+            state = gs.dumps(indent=2)
+        finally:
+            os.chdir(start_dir)
+
         with open(path, 'w') as out:
             out.write(state)
 
@@ -112,10 +138,21 @@ class Application(HubListener):
         """
         from glue.core.state import GlueUnSerializer
 
-        with open(path) as infile:
-            state = GlueUnSerializer.load(infile)
+        # In case relative paths are needed in the session file, we do the
+        # loading while setting the current directory to the directory
+        # in which the session file is so that relative paths are interpreted
+        # as relative to the session file.
+        start_dir = os.path.abspath('.')
+        session_dir = os.path.dirname(path) or '.'
+        session_file = os.path.basename(path)
 
-        return state.object('__main__')
+        try:
+            os.chdir(session_dir)
+            with open(session_file) as infile:
+                state = GlueUnSerializer.load(infile)
+            return state.object('__main__')
+        finally:
+            os.chdir(start_dir)
 
     def new_tab(self):
         raise NotImplementedError()
@@ -149,9 +186,32 @@ class Application(HubListener):
             yield key, value
 
     @catch_error("Could not load data")
-    def load_data(self, path):
-        d = load_data(path)
-        self.add_datasets(self.data_collection, d)
+    def load_data(self, paths, skip_merge=False, auto_merge=False):
+        """
+        Given a path to a file, load the file as a Data object and add it to
+        the current session.
+
+        This returns the added `Data` object.
+        """
+
+        if isinstance(paths, string_types):
+            paths = [paths]
+
+        datasets = []
+        for path in paths:
+            result = load_data(path)
+            if isinstance(result, Data):
+                datasets.append(result)
+            else:
+                datasets.extend(result)
+
+        self.add_datasets(self.data_collection, datasets,
+                          skip_merge=skip_merge, auto_merge=auto_merge)
+
+        if len(datasets) == 1:
+            return datasets[0]
+        else:
+            return datasets
 
     @catch_error("Could not add data")
     def add_data(self, *args, **kwargs):
@@ -161,6 +221,8 @@ class Application(HubListener):
         Positional arguments are interpreted using the data factories, while
         keyword arguments are interpreted using the same infrastructure as the
         `qglue` command.
+
+        This returns a list of added `Data` objects.
         """
 
         datasets = []
@@ -180,6 +242,8 @@ class Application(HubListener):
         if links is not None:
             self.data_collection.add_link(parse_links(self.data_collection, links))
 
+        return datasets
+
     def report_error(self, message, detail):
         """ Report an error message to the user.
         Must be implemented in a subclass
@@ -191,10 +255,10 @@ class Application(HubListener):
         detail : str
             Longer context about the error
         """
-        raise NotImplementedError()
+        raise Exception(message + "(" + detail + ")")
 
     def do(self, command):
-        self._cmds.do(command)
+        return self._cmds.do(command)
 
     def undo(self):
         try:
@@ -212,7 +276,7 @@ class Application(HubListener):
         raise NotImplementedError()
 
     @classmethod
-    def add_datasets(cls, data_collection, datasets):
+    def add_datasets(cls, data_collection, datasets, skip_merge=False, auto_merge=False):
         """ Utility method to interactively add datasets to a
         data_collection
 
@@ -225,7 +289,7 @@ class Application(HubListener):
         Adds datasets to the collection
         """
 
-        datasets = as_list(datasets)
+        datasets = as_flat_data_list(datasets)
         data_collection.extend(datasets)
 
         # We now check whether any of the datasets can be merged. We need to
@@ -245,10 +309,13 @@ class Application(HubListener):
                      if d.shape == shp and d is not data]
 
             # If no other datasets have the same shape, we go to the next one
-            if not other:
+            if not other or skip_merge:
                 continue
 
-            merges, label = cls._choose_merge(data, other)
+            if auto_merge:
+                merges, label = [data] + other, data.label
+            else:
+                merges, label = cls._choose_merge(data, other)
 
             if merges:
                 data_collection.merge(*merges, label=label)
@@ -320,6 +387,9 @@ class ViewerBase(HubListener, PropertySetMixin):
         self._hub = None
         self._layer_artist_container = self._layer_artist_container_cls()
 
+    def close(self):
+        self._layer_artist_container.clear()
+
     def register_to_hub(self, hub):
         self._hub = hub
 
@@ -371,8 +441,7 @@ class ViewerBase(HubListener, PropertySetMixin):
         roi : :class:`~glue.core.roi.Roi`
             The ROI to apply.
         """
-        cmd = command.ApplyROI(client=self.client, roi=roi)
-        self._session.command_stack.do(cmd)
+        raise NotImplementedError
 
     @property
     def session(self):
