@@ -1,13 +1,16 @@
 from __future__ import absolute_import, division, print_function
 
+from contextlib import contextmanager
+
 from glue.core.util import disambiguate
 from glue.core.message import (DataCollectionAddMessage,
                                DataCollectionDeleteMessage,
-                               DataAddComponentMessage)
+                               ComponentsChangedMessage)
 from glue.core.registry import Registry
 from glue.core.link_manager import LinkManager
 from glue.core.data import Data
 from glue.core.hub import Hub, HubListener
+from glue.core.coordinates import WCSCoordinates
 from glue.config import settings
 from glue.utils import as_list
 
@@ -34,7 +37,8 @@ class DataCollection(HubListener):
         :param data: :class:`~glue.core.data.Data` object, or list of such objects
         """
         super(DataCollection, self).__init__()
-        self._link_manager = LinkManager()
+
+        self._link_manager = LinkManager(self)
         self._data = []
 
         self.hub = None
@@ -44,6 +48,8 @@ class DataCollection(HubListener):
         self.extend(as_list(data or []))
         self._sg_count = 0
 
+        self._link_manager.register_to_hub(self.hub)
+
     @property
     def data(self):
         """ The :class:`~glue.core.data.Data` objects in the collection """
@@ -58,6 +64,7 @@ class DataCollection(HubListener):
 
         :param data: :class:`~glue.core.data.Data` object to add
         """
+
         if isinstance(data, list):
             self.extend(data)
             return
@@ -70,6 +77,7 @@ class DataCollection(HubListener):
                 s.register()
             msg = DataCollectionAddMessage(self, data)
             self.hub.broadcast(msg)
+
         self._sync_link_manager()
 
     def extend(self, data):
@@ -79,7 +87,11 @@ class DataCollection(HubListener):
 
         :param data: List of data objects to add
         """
-        [self.append(d) for d in data]
+        # Wait until all datasets are added to sync the link manager
+        with self._no_sync_link_manager():
+            for d in data:
+                self.append(d)
+        self._sync_link_manager()
 
     def remove(self, data):
         """ Remove a data set from the collection
@@ -102,15 +114,18 @@ class DataCollection(HubListener):
         for each data set are up-to-date
         """
 
-        # add any links in the data
-        for d in self._data:
-            for derived in d.derived_components:
-                self._link_manager.add_link(d.get_component(derived).link)
-            for link in d.coordinate_links:
-                self._link_manager.add_link(link)
+        if getattr(self, '_disable_sync_link_manager', False):
+            return
 
-        for d in self._data:
-            self._link_manager.update_data_components(d)
+        # Avoid circular calls
+        with self._no_sync_link_manager():
+            self._link_manager.update_externally_derivable_components()
+
+    @contextmanager
+    def _no_sync_link_manager(self):
+        self._disable_sync_link_manager = True
+        yield
+        self._disable_sync_link_manager = False
 
     @property
     def links(self):
@@ -119,6 +134,13 @@ class DataCollection(HubListener):
         """
         return tuple(self._link_manager.links)
 
+    @property
+    def external_links(self):
+        """
+        Tuple of :class:`~glue.core.component_link.ComponentLink` objects.
+        """
+        return tuple(self._link_manager.external_links)
+
     def add_link(self, links):
         """Add one or more links to the data collection.
 
@@ -130,25 +152,33 @@ class DataCollection(HubListener):
            instances, or a :class:`~glue.core.link_helpers.LinkCollection`
         """
         self._link_manager.add_link(links)
-        for d in self._data:
-            self._link_manager.update_data_components(d)
+
+    def remove_link(self, links):
+        """
+        Remove one or more links from the data collection.
+
+        This will auto-update the components in each data set
+
+        :param links:
+           The links to remove. A scalar or list of
+           :class:`~glue.core.component_link.ComponentLink`
+           instances, or a :class:`~glue.core.link_helpers.LinkCollection`
+        """
+        self._link_manager.remove_link(links)
 
     def _merge_link(self, link):
         pass
 
     def set_links(self, links):
-        """Override the links in the collection, and update data
-        objects as necessary.
+        """
+        Override the links in the collection, and update data objects as
+        necessary.
 
         :param links: The new links. An iterable of
             :class:`~glue.core.component_link.ComponentLink` instances
         """
-        self._link_manager.clear()
-        for link in links:
-            self._link_manager.add_link(link)
-
-        for d in self._data:
-            self._link_manager.update_data_components(d)
+        self._link_manager.clear_links()
+        self._link_manager.add_link(links)
 
     def register_to_hub(self, hub):
         """ Register managed data objects to a hub.
@@ -172,7 +202,7 @@ class DataCollection(HubListener):
             for s in d.subsets:
                 s.register()
 
-        hub.subscribe(self, DataAddComponentMessage,
+        hub.subscribe(self, ComponentsChangedMessage,
                       lambda msg: self._sync_link_manager(),
                       filter=lambda x: x.sender in self._data)
 
@@ -190,7 +220,7 @@ class DataCollection(HubListener):
         from glue.core.subset_group import SubsetGroup
         color = settings.SUBSET_COLORS[self._sg_count % len(settings.SUBSET_COLORS)]
         self._sg_count += 1
-        label = label or "%i" % (self._sg_count)
+        label = label or 'Subset %i' % self._sg_count
 
         result = SubsetGroup(color=color, label=label, subset_state=subset_state)
         self._subset_groups.append(result)
@@ -234,15 +264,23 @@ class DataCollection(HubListener):
             if d.shape != shp:
                 raise ValueError("All arguments must have the same shape")
 
-
         label = kwargs.get('label', data[0].label)
 
         master = Data(label=label)
         self.append(master)
 
+        master.coords = data[0].coords
+        for i, d in enumerate(data):
+            if isinstance(d.coords, WCSCoordinates):
+                master.coords = d.coords
+                break
+
         for d in data:
+
             skip = d.pixel_component_ids + d.world_component_ids
+
             for c in d.components:
+
                 if c in skip:
                     continue
 
@@ -262,7 +300,9 @@ class DataCollection(HubListener):
 
                 lbl = disambiguate(lbl, taken)
                 c._label = lbl
+                c.parent = master
                 master.add_component(d.get_component(c), c)
+
             self.remove(d)
 
         return self
@@ -286,6 +326,9 @@ class DataCollection(HubListener):
     def __len__(self):
         return len(self._data)
 
+    def index(self, item):
+        return self._data.index(item)
+
     def __str__(self):
         if len(self) == 1:
             result = "DataCollection (1 data set)\n\t"
