@@ -7,7 +7,7 @@ from collections import OrderedDict
 from glue.core.coordinates import coordinates_from_header, WCSCoordinates
 from glue.core.data import Component, Data
 from glue.config import data_factory, qglue_parser
-
+from glue.external.six import string_types
 
 __all__ = ['is_fits', 'fits_reader', 'is_casalike', 'casalike_cube']
 
@@ -52,11 +52,15 @@ def fits_reader(source, auto_merge=False, exclude_exts=None, label=None):
     from astropy.table import Table
 
     exclude_exts = exclude_exts or []
-    if not isinstance(source, fits.hdu.hdulist.HDUList):
+
+    if isinstance(source, fits.hdu.hdulist.HDUList):
+        hdulist = source
+        close_hdulist = False
+    else:
         hdulist = fits.open(source, ignore_missing_end=True)
         hdulist.verify('fix')
-    else:
-        hdulist = source
+        close_hdulist = True
+
     groups = OrderedDict()
     extension_by_shape = OrderedDict()
 
@@ -83,6 +87,20 @@ def fits_reader(source, auto_merge=False, exclude_exts=None, label=None):
         )
         data = Data(label=label)
         data.coords = coords
+
+        # We need to be careful here because some header values are special
+        # objects that we should convert to strings
+        for key, value in hdu.header.items():
+            if (key == 'COMMENT' or key == 'HISTORY'):
+                if key not in data.meta:
+                    data.meta[key] = [str(value)]
+                else:
+                    data.meta[key].append(str(value))
+            elif isinstance(value, string_types) or isinstance(value, (int, float, bool)):
+                data.meta[key] = value
+            else:
+                data.meta[key] = str(value)
+
         groups[hdu_name] = data
         extension_by_shape[shape] = hdu_name
         return data
@@ -108,10 +126,7 @@ def fits_reader(source, auto_merge=False, exclude_exts=None, label=None):
             elif is_table_hdu(hdu):
                 # Loop through columns and make component list
                 table = Table.read(hdu, format='fits')
-                label = '{0}[{1}]'.format(
-                    label_base,
-                    hdu_name
-                )
+                label = '{0}[{1}]'.format(label_base, hdu_name)
                 data = Data(label=label)
                 groups[hdu_name] = data
                 for column_name in table.columns:
@@ -122,6 +137,10 @@ def fits_reader(source, auto_merge=False, exclude_exts=None, label=None):
                     component = Component.autotyped(column, units=column.unit)
                     data.add_component(component=component,
                                        label=column_name)
+
+    if close_hdulist:
+        hdulist.close()
+
     return [groups[idx] for idx in groups]
 
 
@@ -139,8 +158,8 @@ def is_table_hdu(hdu):
 
 def has_wcs(coords):
     return (isinstance(coords, WCSCoordinates) and
-               any(axis['coordinate_type'] is not None
-               for axis in coords.wcs.get_axis_types()))
+            any(axis['coordinate_type'] is not None
+                for axis in coords.wcs.get_axis_types()))
 
 
 def is_casalike(filename, **kwargs):
