@@ -32,7 +32,7 @@ from glue.core.data import Component, Data
 from glue.config import auto_refresh, data_factory
 from glue.backends import get_timer
 from glue.utils import as_list
-
+from glue.logger import logger
 
 __all__ = ['FileWatcher', 'LoadLog',
            'auto_data', 'data_label', 'find_factory',
@@ -76,7 +76,7 @@ def has_extension(exts):
     """
 
     def tester(x, **kwargs):
-        return _extension(x) in set(exts.split())
+        return _extension(x).lower() in set(exts.lower().split())
     return tester
 
 
@@ -104,6 +104,8 @@ class LoadLog(object):
         else:
             self.watcher = None
 
+        self._absolute = True
+
     def _log_component(self, component):
         self.components.append(component)
 
@@ -144,20 +146,26 @@ class LoadLog(object):
 
             mapping = dict((c, log.component(self.id(c)).data)
                            for c in dold._components.values()
-                           if c in self.components
-                           and type(c) == Component)
+                           if c in self.components and
+                           type(c) == Component)
             dold.coords = dnew.coords
             dold.update_components(mapping)
 
     def __gluestate__(self, context):
-        return dict(path=self.path,
+        if context.absolute_paths:
+            path = os.path.abspath(self.path)
+        else:
+            path = os.path.relpath(self.path)
+        return dict(path=path,
                     factory=context.do(self.factory),
-                    kwargs=[list(self.kwargs.items())])
+                    kwargs=[list(self.kwargs.items())],
+                    _protocol=1)
 
     @classmethod
     def __setgluestate__(cls, rec, context):
         fac = context.object(rec['factory'])
         kwargs = dict(*rec['kwargs'])
+        kwargs['coord_first'] = rec.get('_protocol', 0) >= 1
         d = load_data(rec['path'], factory=fac, **kwargs)
         return as_list(d)[0]._load_log
 
@@ -221,6 +229,8 @@ def load_data(path, factory=None, **kwargs):
     """
     from glue.qglue import parse_data
 
+    coord_first = kwargs.pop('coord_first', True)
+
     def as_data_objects(ds, lbl):
         # pack other container types like astropy tables
         # into glue data objects
@@ -231,18 +241,36 @@ def load_data(path, factory=None, **kwargs):
             for item in parse_data(d, lbl):
                 yield item
 
-    factory = factory or auto_data
+    if factory is None:
+        factory = find_factory(path, **kwargs)
+        if factory is None:
+            raise KeyError("Don't know how to open file: %s" % path)
+
     lbl = data_label(path)
 
     d = as_list(factory(path, **kwargs))
     d = list(as_data_objects(d, lbl))
     log = LoadLog(path, factory, kwargs)
     for item in d:
-        if item.label is '':
+        if not item.label:
             item.label = lbl
         log.log(item)  # attaches log metadata to item
-        for cid in item.primary_components:
+
+        if coord_first:
+            # We just follow the order in which the components are now loaded,
+            # which is coordinate components first, followed by all other
+            # components
+            for cid in item.primary_components:
+                log.log(item.get_component(cid))
+        else:
+            # In this case the first component was the first one that is not a
+            # coordinate component, followed by the coordinate components,
+            # followed by the remaining components.
+            cid = item.primary_components[item.ndim * 2]
             log.log(item.get_component(cid))
+            for icid, cid in enumerate(item.primary_components):
+                if icid != item.ndim * 2:
+                    log.log(item.get_component(cid))
 
     if len(d) == 1:
         # unpack single-length lists for user convenience
@@ -290,6 +318,8 @@ def find_factory(filename, **kwargs):
 
     for df in data_factory:
 
+        logger.info('Trying data factory {0}'.format(df.label))
+
         # Once we've found a match, and iterated through the rest of the
         # importers with the same priority, we can exit the loop.
         if best_priority is not None and df.priority < best_priority:
@@ -309,6 +339,8 @@ def find_factory(filename, **kwargs):
             valid_formats.append(df)
             best_priority = df.priority
 
+    logger.info('Valid formats: {0}'.format(valid_formats))
+
     if len(valid_formats) == 0:
         return None
     elif len(valid_formats) > 1:
