@@ -1,12 +1,14 @@
 from __future__ import absolute_import, division, print_function
 
 import logging
+from contextlib import contextmanager
+from weakref import WeakKeyDictionary
 from inspect import getmro
-from collections import defaultdict
+from collections import Counter
 
 from glue.core.exceptions import InvalidSubscriber, InvalidMessage
 from glue.core.message import Message
-
+from glue.core.hub_callback_container import HubCallbackContainer
 
 __all__ = ['Hub', 'HubListener']
 
@@ -37,7 +39,12 @@ class Hub(object):
         to the new hub object.
         """
         # Dictionary of subscriptions
-        self._subscriptions = defaultdict(dict)
+        self._subscriptions = WeakKeyDictionary()
+
+        self._paused = False
+        self._queue = []
+
+        self._ignore = Counter()
 
         from glue.core.data import Data
         from glue.core.subset import Subset
@@ -76,14 +83,12 @@ class Hub(object):
            An optional function of the form handler(message) that will
            receive the message on behalf of the subscriber. If not provided,
            this defaults to the HubListener's notify method
-        :type handler: Callable
 
 
         :param filter:
            An optional function of the form filter(message). Messages
            are only passed to the subscriber if filter(message) == True.
            The default is to always pass messages.
-        :type filter: Callable
 
 
         Raises:
@@ -107,7 +112,10 @@ class Hub(object):
         if not handler:
             handler = subscriber.notify
 
-        self._subscriptions[subscriber][message_class] = (filter, handler)
+        if subscriber not in self._subscriptions:
+            self._subscriptions[subscriber] = HubCallbackContainer()
+
+        self._subscriptions[subscriber][message_class] = handler, filter
 
     def is_subscribed(self, subscriber, message):
         """
@@ -121,12 +129,14 @@ class Hub(object):
             True if the subscriber/message pair have been subscribed to the hub
 
         """
-        return subscriber in self._subscriptions and \
-            message in self._subscriptions[subscriber]
+        return (subscriber in self._subscriptions and
+                message in self._subscriptions[subscriber])
 
     def get_handler(self, subscriber, message):
+        if subscriber is None:
+            return None
         try:
-            return self._subscriptions[subscriber][message][1]
+            return self._subscriptions[subscriber][message][0]
         except KeyError:
             return None
 
@@ -160,25 +170,51 @@ class Hub(object):
             # subscriptions to message or its superclasses
             messages = [msg for msg in subscriptions.keys() if
                         issubclass(type(message), msg)]
+
             if len(messages) == 0:
                 continue
 
             # narrow to the most-specific message
             candidate = max(messages, key=_mro_count)
 
-            test, handler = subscriptions[candidate]
+            handler, test = subscriptions[candidate]
             if test(message):
                 yield subscriber, handler
 
+    @contextmanager
+    def ignore_callbacks(self, ignore_type):
+        self._ignore[ignore_type] += 1
+        try:
+            yield
+        finally:
+            self._ignore[ignore_type] -= 1
+
+    @contextmanager
+    def delay_callbacks(self):
+        self._paused = True
+        try:
+            yield
+        finally:
+            self._paused = False
+            # TODO: could de-duplicate messages here
+            for message in self._queue:
+                self.broadcast(message)
+            self._queue = []
+
     def broadcast(self, message):
         """Broadcasts a message to all subscribed objects.
 
         :param message: The message to broadcast
         :type message: :class:`~glue.core.message.Message`
         """
-        logging.getLogger(__name__).info("Broadcasting %s", message)
-        for subscriber, handler in self._find_handlers(message):
-            handler(message)
+        if self._ignore.get(type(message), 0) > 0:
+            return
+        elif self._paused:
+            self._queue.append(message)
+        else:
+            logging.getLogger(__name__).info("Broadcasting %s", message)
+            for subscriber, handler in self._find_handlers(message):
+                handler(message)
 
     def __getstate__(self):
         """ Return a picklable representation of the hub
