@@ -11,6 +11,9 @@ these layers, and provides GUI access to the model
 
 from __future__ import absolute_import, division, print_function
 
+import textwrap
+from weakref import WeakKeyDictionary
+
 from qtpy.QtCore import Qt
 from qtpy import QtCore, QtWidgets, QtGui
 from glue.core.layer_artist import LayerArtistBase, LayerArtistContainer
@@ -19,6 +22,8 @@ from glue.icons.qt import layer_artist_icon
 from glue.core.qt.mime import LAYERS_MIME_TYPE
 from glue.utils import nonpartial
 from glue.utils.qt import PythonListModel, PyMimeData
+from glue.core.hub import HubListener
+from glue.core.message import LayerArtistEnabledMessage, LayerArtistUpdatedMessage, LayerArtistDisabledMessage
 
 
 class LayerArtistModel(PythonListModel):
@@ -38,7 +43,7 @@ class LayerArtistModel(PythonListModel):
 
     def data(self, index, role):
         """Retrieve data at each index"""
-        if not index.isValid():
+        if not index.isValid() or index.row() >= len(self.artists):
             return None
         if role == Qt.DecorationRole:
             art = self.artists[index.row()]
@@ -46,22 +51,27 @@ class LayerArtistModel(PythonListModel):
             return result
         if role == Qt.CheckStateRole:
             art = self.artists[index.row()]
-            result = Qt.Checked if art.visible else Qt.Unchecked
+            result = Qt.Checked if art.visible and art.enabled else Qt.Unchecked
             return result
         if role == Qt.ToolTipRole:
             art = self.artists[index.row()]
             if not art.enabled:
-                return art.disabled_message
+                wrapped = textwrap.fill(art.disabled_message, break_long_words=False)
+                return wrapped
 
         return super(LayerArtistModel, self).data(index, role)
 
     def flags(self, index):
         result = super(LayerArtistModel, self).flags(index)
-        if index.isValid():
-            result = (result | Qt.ItemIsEditable | Qt.ItemIsDragEnabled |
-                      Qt.ItemIsUserCheckable)
+        if index.isValid() and index.row() < len(self.artists):
+            art = self.artists[index.row()]
+            if art.enabled:
+                result = (result | Qt.ItemIsEditable | Qt.ItemIsDragEnabled |
+                          Qt.ItemIsUserCheckable)
+            else:
+                result = (result & Qt.ItemIsUserCheckable) ^ result
         else:  # only drop between rows, where index isn't valid
-            result = (result | Qt.ItemIsDropEnabled)
+            result = result | Qt.ItemIsDropEnabled
 
         return result
 
@@ -80,7 +90,7 @@ class LayerArtistModel(PythonListModel):
 
     def _remove_row(self, row):
         art = self.artists.pop(row)
-        art.clear()
+        art.remove()
         art.redraw()
 
     def mimeTypes(self):
@@ -143,7 +153,10 @@ class LayerArtistModel(PythonListModel):
 
     def row_label(self, row):
         """ The textual label for the row"""
-        layer = self.artists[row].layer
+        artist = self.artists[row]
+        if hasattr(artist, 'label'):
+            return artist.label
+        layer = artist.layer
         if hasattr(layer, 'verbose_label'):
             return layer.verbose_label
         return layer.label
@@ -166,14 +179,17 @@ class LayerArtistModel(PythonListModel):
     def row_artist(self, row):
         return self.artists[row]
 
+    def artist_row(self, artist):
+        return self.artists.index(artist)
 
-class LayerArtistView(QtWidgets.QListView):
+
+class LayerArtistView(QtWidgets.QListView, HubListener):
 
     """A list view into an artist model. The zorder
     of each artist can be shuffled by dragging and dropping
     items. Right-clicking brings up a menu to edit style or delete"""
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, hub=None):
         super(LayerArtistView, self).__init__(parent)
         self.setDragEnabled(True)
         self.setAcceptDrops(True)
@@ -184,19 +200,38 @@ class LayerArtistView(QtWidgets.QListView):
         self.setContextMenuPolicy(Qt.ActionsContextMenu)
         self.setEditTriggers(self.NoEditTriggers)
 
-        self._set_palette()
+        self.setMinimumSize(200, 100)
         self._actions = {}
         self._create_actions()
 
-        self._timer = QtCore.QTimer(self)
-        self._timer.timeout.connect(nonpartial(self._update_viewport))
-        self._timer.start(1000)
+        # Update when any message is emitted which would indicate a change in
+        # data collection content, colors, labels, etc. It's easier to simply
+        # listen to all events since the viewport update is fast.
+        self.hub = hub
+        self.hub.subscribe(self, LayerArtistUpdatedMessage, self._update_viewport)
+        self.hub.subscribe(self, LayerArtistEnabledMessage, self._layer_enabled_or_disabled)
+        self.hub.subscribe(self, LayerArtistDisabledMessage, self._layer_enabled_or_disabled)
 
-    def _update_viewport(self):
-        # We have to do this here to make sure we always get the latest
-        # viewport instance.
+    def _update_viewport(self, *args):
+        # This forces the widget containing the list view to update/redraw,
+        # reflecting any changes in color/labels/content
         self.viewport().update()
 
+    def _layer_enabled_or_disabled(self, *args):
+
+        # This forces the widget containing the list view to update/redraw,
+        # reflecting any changes in disabled/enabled layers. If a layer is
+        # disabled, it will become unselected.
+        self.viewport().update()
+
+        # If a layer artist becomes unselected as a result of the update above,
+        # a selection change event is not emitted for some reason, so we force
+        # a manual update. If a layer artist was deselected, current_artist()
+        # will be None and the options will be hidden for that layer.
+        parent = self.parent()
+        if parent is not None:
+            parent.on_selection_change(self.current_artist())
+
     def rowsInserted(self, index, start, end):
         super(LayerArtistView, self).rowsInserted(index, start, end)
         # If no rows are currently selected, make sure we select one. We do
@@ -210,7 +245,6 @@ class LayerArtistView(QtWidgets.QListView):
 
     def selectionChanged(self, selected, deselected):
         super(LayerArtistView, self).selectionChanged(selected, deselected)
-        self._update_actions()
         parent = self.parent()
         if parent is not None:
             parent.on_selection_change(self.current_artist())
@@ -224,6 +258,11 @@ class LayerArtistView(QtWidgets.QListView):
             return
         return self.model().row_artist(rows[0].row())
 
+    def select_artist(self, artist):
+        model = self.selectionModel()
+        row = self.model().artist_row(artist)
+        model.select(model.model().index(row), model.ClearAndSelect)
+
     def single_selection(self):
         return self.current_artist() is not None
 
@@ -236,16 +275,6 @@ class LayerArtistView(QtWidgets.QListView):
             return
         return rows[0].row()
 
-    def _set_palette(self):
-        p = self.palette()
-        c = QtGui.QColor(240, 240, 240)
-        p.setColor(QtGui.QPalette.Highlight, c)
-        p.setColor(QtGui.QPalette.HighlightedText, QtGui.QColor(Qt.black))
-        self.setPalette(p)
-
-    def _update_actions(self):
-        pass
-
     def _bottom_left_of_current_index(self):
         idx = self.currentIndex()
         if not idx.isValid():
@@ -281,16 +310,18 @@ class LayerArtistWidget(QtWidgets.QWidget):
     options for the layer artists.
     """
 
-    def __init__(self, parent=None, layer_style_widget_cls=None):
+    def __init__(self, parent=None, layer_style_widget_cls=None, hub=None):
 
         super(LayerArtistWidget, self).__init__(parent=parent)
 
+        self.hub = None
+
         self.layout = QtWidgets.QVBoxLayout()
         self.layout.setContentsMargins(0, 0, 0, 0)
 
         self.layer_style_widget_cls = layer_style_widget_cls
 
-        self.layer_list = LayerArtistView(parent=self)
+        self.layer_list = LayerArtistView(parent=self, hub=hub)
         self.layout.addWidget(self.layer_list)
 
         self.layer_options = QtWidgets.QWidget()
@@ -301,11 +332,22 @@ class LayerArtistWidget(QtWidgets.QWidget):
 
         self.setLayout(self.layout)
 
-        self.layout_style_widgets = {}
+        self.layout_style_widgets = WeakKeyDictionary()
 
         self.empty = QtWidgets.QWidget()
         self.layer_options_layout.addWidget(self.empty)
 
+        self.disabled_warning = QtWidgets.QLabel()
+        self.disabled_warning.setWordWrap(True)
+        self.disabled_warning.setAlignment(Qt.AlignJustify)
+        self.padded_warning = QtWidgets.QWidget()
+        warning_layout = QtWidgets.QVBoxLayout()
+        warning_layout.setContentsMargins(20, 20, 20, 20)
+        warning_layout.addWidget(self.disabled_warning)
+        self.padded_warning.setLayout(warning_layout)
+
+        self.layer_options_layout.addWidget(self.padded_warning)
+
     def on_artist_add(self, layer_artists):
 
         if self.layer_style_widget_cls is None:
@@ -326,14 +368,16 @@ class LayerArtistWidget(QtWidgets.QWidget):
 
     def on_selection_change(self, layer_artist):
 
-        if layer_artist is None:
-            return
-
         if layer_artist in self.layout_style_widgets:
-            self.layer_options_layout.setCurrentWidget(self.layout_style_widgets[layer_artist])
+            if layer_artist.enabled:
+                self.layer_options_layout.setCurrentWidget(self.layout_style_widgets[layer_artist])
+                self.disabled_warning.setText('')
+            else:
+                self.disabled_warning.setText(layer_artist.disabled_message)
+                self.layer_options_layout.setCurrentWidget(self.padded_warning)
         else:
             self.layer_options_layout.setCurrentWidget(self.empty)
-
+            self.disabled_warning.setText('')
 
 
 class QtLayerArtistContainer(LayerArtistContainer):
@@ -349,21 +393,18 @@ class QtLayerArtistContainer(LayerArtistContainer):
         self.model.modelReset.connect(nonpartial(self._notify))
 
     def append(self, artist):
-        self._check_duplicate(artist)
         self.model.add_artist(0, artist)
         artist.zorder = max(a.zorder for a in self.artists) + 1
         assert self.artists[0] is artist
         self._notify()
 
     def remove(self, artist):
-        try:
+        if artist in self.artists:
             index = self.artists.index(artist)
-        except ValueError:
-            return
-        self.model.removeRow(index)
-        assert artist not in self.artists
-
-        self._notify()
+            self.model.removeRow(index)
+            assert artist not in self.artists
+            artist.remove()
+            self._notify()
 
     def __nonzero__(self):
         return True
