@@ -4,16 +4,19 @@ from __future__ import absolute_import, division, print_function
 
 import pytest
 import numpy as np
+from numpy.testing import assert_equal
 from mock import MagicMock
 
 from glue.external import six
 from glue import core
 
-from ..component import Component, DerivedComponent, CategoricalComponent
+from ..component import Component, DerivedComponent, CategoricalComponent, DateTimeComponent
 from ..component_id import ComponentID
-from ..component_link import ComponentLink
+from ..component_link import ComponentLink, CoordinateComponentLink, BinaryComponentLink
 from ..coordinates import Coordinates
 from ..data import Data, pixel_label
+from ..link_helpers import LinkSame
+from ..data_collection import DataCollection
 from ..exceptions import IncompatibleAttribute
 from ..hub import Hub
 from ..registry import Registry
@@ -23,6 +26,8 @@ from ..subset import (Subset, CategoricalROISubsetState, SubsetState,
                       CategoricalROISubsetState2D, AndState)
 from ..roi import PolygonalROI, CategoricalROI, RangeROI, RectangularROI
 
+from .test_state import clone
+
 
 class _TestCoordinates(Coordinates):
 
@@ -93,7 +98,7 @@ class TestData(object):
             assert exc.value.args[0] == ("add_component() missing 1 required "
                                          "positional argument: 'label'")
         else:
-            assert exc.value.args[0] == ("add_component() takes at least 3 "
+            assert exc.value.args[0] == ("add_component() takes exactly 3 "
                                          "arguments (2 given)")
 
     def test_get_getitem_incompatible_attribute(self):
@@ -259,8 +264,11 @@ class TestData(object):
         assert hub.broadcast.call_count == 1
 
     def test_remove_component(self):
+        hub = MagicMock(spec_set=Hub)
+        self.data.register_to_hub(hub)
         self.data.remove_component(self.comp_id)
-        assert not self.comp_id in self.data.components
+        assert self.comp_id not in self.data.components
+        assert hub.broadcast.call_count == 2
 
     def test_get_component(self):
         assert self.data.get_component(self.comp_id) is self.comp
@@ -347,32 +355,101 @@ class TestData(object):
         d.update_components({d.id['x']: [10, 20, 30]})
         np.testing.assert_array_equal(s.to_mask(), [False, False, False])
 
+    def test_add_derived_implicit(self):
+
+        # Regression test for a bug that caused derived components added via
+        # the data[...] = ... syntax to have links that did not include a 'to'
+        # argument, leading the link manager to add a ghost component to the
+        # data.
+
+        from ..data_collection import DataCollection
+
+        dc = DataCollection([])
+
+        data = Data(x=[1, 2, 3], y=[2, 3, 4], label='data1')
+        dc.append(data)
+
+        data['z'] = data.id['x'] + 1
+
+        # There should be five components: x, y, z, pixel, and world
+        assert len(data.components) == 5
+
+    def test_remove_derived_dependency(self):
+
+        # Regression test for a bug that occurred when removing a component
+        # used in a derived component, which should also remove the derived
+        # component itself. To make things more fun, we set up a chain of
+        # derived components to make sure they are all removed.
+
+        data = Data(a=[1, 2, 3], b=[2, 3, 4], label='data1')
+
+        data['c'] = data.id['a'] + 1
+        data['d'] = data.id['c'] + 1
+        data['e'] = data.id['d'] + 1
+        data['f'] = data.id['e'] + 1
+
+        a_id = data.id['a']
+        b_id = data.id['b']
+        c_id = data.id['c']
+        d_id = data.id['d']
+        e_id = data.id['e']
+        f_id = data.id['f']
+
+        # There should be five components: pixel, world, a, b, c, d, e, f
+        assert len(data.components) == 8
+
+        data.remove_component(data.id['d'])
+
+        # This should also remove e and f since they depend on d
+
+        assert len(data.components) == 5
+
+        assert a_id in data.components
+        assert b_id in data.components
+        assert c_id in data.components
+        assert d_id not in data.components
+        assert e_id not in data.components
+        assert f_id not in data.components
+
+    def test_links_property(self):
+
+        data = Data(a=[1, 2, 3], b=[2, 3, 4], label='data1')
+
+        assert len(data.links) == 2
+        assert isinstance(data.links[0], CoordinateComponentLink)
+        assert isinstance(data.links[1], CoordinateComponentLink)
+
+        data['c'] = data.id['a'] + 1
+
+        assert len(data.links) == 3
+
+        assert isinstance(data.links[2], BinaryComponentLink)
+
+
 class TestROICreation(object):
 
     def test_range_roi(self):
 
         d = Data(xdata=[1, 2, 3], ydata=[1, 2, 3])
         comp = d.get_component(d.id['xdata'])
-        roi = RangeROI('x', min=2,max=3)
+        roi = RangeROI('x', min=2, max=3)
         s = comp.subset_from_roi('xdata', roi)
         assert isinstance(s, RangeSubsetState)
         np.testing.assert_array_equal((s.lo, s.hi),
                                       [2, 3])
 
-        roi = RangeROI('y', min=2,max=3)
+        roi = RangeROI('y', min=2, max=3)
         s = comp.subset_from_roi('xdata', roi, other_att='ydata',
                                  other_comp=d.get_component(d.id['ydata']))
         assert isinstance(s, RangeSubsetState)
         assert s.att == 'ydata'
 
-
-
     def test_range_roi_categorical(self):
 
         d = Data(x=['a', 'b', 'c'], y=[1, 2, 3])
         comp = d.get_component(d.id['x'])
         roi = CategoricalROI(['b', 'c'])
-        s = comp.subset_from_roi('x', roi)
+        s = comp.subset_from_roi(d.id['x'], roi)
         assert isinstance(s, CategoricalROISubsetState)
         np.testing.assert_array_equal((s.roi.contains(['a', 'b', 'c'], None)),
                                       [False, True, True])
@@ -389,7 +466,7 @@ class TestROICreation(object):
         x_comp = d.get_component(d.id['x'])
         y_comp = d.get_component(d.id['y'])
         roi = PolygonalROI([0, 0, 2, 2], [0, 2, 2, 0])
-        s = x_comp.subset_from_roi('x', roi, other_comp=y_comp, other_att='y')
+        s = x_comp.subset_from_roi(d.id['x'], roi, other_comp=y_comp, other_att=d.id['y'])
         assert isinstance(s, RoiSubsetState)
         np.testing.assert_array_equal(s.to_mask(d), [True, True, False, False])
 
@@ -529,8 +606,6 @@ def test_foreign_pixel_components_not_in_visible():
     """Pixel components from other data should not be visible"""
 
     # currently, this is trivially satisfied since all coordinates are hidden
-    from ..link_helpers import LinkSame
-    from ..data_collection import DataCollection
 
     d1 = Data(x=[1], y=[2])
     d2 = Data(w=[3], v=[4])
@@ -557,46 +632,204 @@ EXPECTED_STR = """
 Data Set: mydata
 Number of dimensions: 1
 Shape: 3
-Components:
- 0) x
- 1) Pixel Axis 0 [x]
- 2) World 0
- 3) y
+Main components:
+ - x
+ - y
+Coordinate components:
+ - Pixel Axis 0 [x]
+ - World 0
 """.strip()
 
+
 def test_data_str():
     # Regression test for Data.__str__
-    d = Data(x=[1,2,3], y=[2,3,4], label='mydata')
+    d = Data(x=[1, 2, 3], y=[2, 3, 4], label='mydata')
     assert str(d) == EXPECTED_STR
 
 
+EXPECTED_STR_WITH_DERIVED = """
+Data Set: mydata
+Number of dimensions: 1
+Shape: 3
+Main components:
+ - x
+ - y
+Derived components:
+ - z
+Coordinate components:
+ - Pixel Axis 0 [x]
+ - World 0
+""".strip()
+
+
+def test_data_str_with_derived():
+    d = Data(x=[1, 2, 3], y=[2, 3, 4], label='mydata')
+    d['z'] = d.id['x'] + 1
+    assert str(d) == EXPECTED_STR_WITH_DERIVED
+
+
 def test_update_values_from_data():
-    d1 = Data(a=[1,2,3], b=[4,5,6], label='banana')
-    d2 = Data(b=[1,2,3,4], c=[5,6,7,8], label='apple')
+    d1 = Data(a=[1, 2, 3], b=[4, 5, 6], label='banana')
+    d2 = Data(b=[1, 2, 3, 4], c=[5, 6, 7, 8], label='apple')
     d1a = d1.id['a']
     d1b = d1.id['b']
     d2b = d2.id['b']
     d2c = d2.id['c']
     d1.update_values_from_data(d2)
-    assert not d1a in d1.components
+    assert d1a not in d1.components
     assert d1b in d1.components
-    assert not d2b in d1.components
-    assert d2c in d1.components
+    assert d2b not in d1.components
+    assert d2c not in d1.components
+    assert [cid.label for cid in d1.visible_components] == ['b', 'c']
     assert d1.shape == (4,)
 
 
 def test_update_values_from_data_invalid():
 
-    d1 = Data(a=[1,2,3], label='banana')
-    d1.add_component([3,4,5], 'a')
-    d2 = Data(b=[1,2,3,4], c=[5,6,7,8], label='apple')
+    d1 = Data(a=[1, 2, 3], label='banana')
+    d1.add_component([3, 4, 5], 'a')
+    d2 = Data(b=[1, 2, 3, 4], c=[5, 6, 7, 8], label='apple')
     with pytest.raises(ValueError) as exc:
         d1.update_values_from_data(d2)
     assert exc.value.args[0] == "Non-unique component labels in original data"
 
-    d1 = Data(a=[1,2,3], b=[4,5,6], label='banana')
-    d2 = Data(b=[1,2,3,4], label='apple')
-    d2.add_component([5,6,7,8], 'b')
+    d1 = Data(a=[1, 2, 3], b=[4, 5, 6], label='banana')
+    d2 = Data(b=[1, 2, 3, 4], label='apple')
+    d2.add_component([5, 6, 7, 8], 'b')
     with pytest.raises(ValueError) as exc:
         d1.update_values_from_data(d2)
     assert exc.value.args[0] == "Non-unique component labels in new data"
+
+
+def test_update_values_from_data_order():
+
+    # Make sure that the order of components is preserved when calling
+    # Data.update_values_from_data. The final order should be first
+    # components that existed before, followed by new components
+
+    d1 = Data()
+    d1['c'] = [1, 2, 3]
+    d1['b'] = [2, 3, 4]
+    d1['j'] = [0, 1, 2]
+    d1['a'] = [4, 4, 4]
+    d1['f'] = [4, 5, 6]
+
+    d2 = Data()
+    d2['h'] = [4, 4, 4]
+    d2['j'] = [0, 1, 2]
+    d2['a'] = [4, 4, 4]
+
+    d2.update_values_from_data(d1)
+
+    assert [cid.label for cid in d2.visible_components] == ['j', 'a', 'c', 'b', 'f']
+
+
+def test_find_component_id_with_cid():
+
+    # Regression test for a bug that caused Data.find_component_id to return
+    # True erroneously when passing a component ID.
+
+    d1 = Data()
+    d1['a'] = ['a', 'b', 'c']
+    d1['b'] = [1, 2, 3]
+
+    assert d1.find_component_id(d1.id['a']) is d1.id['a']
+    assert d1.find_component_id(d1.id['b']) is d1.id['b']
+
+
+def test_parent_preserved_session():
+
+    # Regression test for a bug that caused ComponentID parents to not be
+    # preserved when saving and restoring a session.
+
+    from ..link_helpers import LinkSame
+    from ..data_collection import DataCollection
+
+    d1 = Data(x=[1], y=[2], label='test1')
+    d2 = Data(w=[3], v=[4], label='test2')
+
+    dc = DataCollection([d1, d2])
+    dc.add_link(LinkSame(d1.id['x'], d2.id['w']))
+
+    assert d1.id['x'].parent is d1
+    assert d1.id['y'].parent is d1
+
+    assert d2.id['w'].parent is d2
+    assert d2.id['v'].parent is d2
+
+    dc2 = clone(dc)
+
+    assert dc2[0].id['x'].parent.label == 'test1'
+    assert dc2[0].id['y'].parent.label == 'test1'
+
+    assert dc2[1].id['w'].parent.label == 'test2'
+    assert dc2[1].id['v'].parent.label == 'test2'
+
+
+def test_preserve_datetime():
+    # Make sure that we recognize and preserve the Numpy datetime64 format
+    dates = np.array([1, 2, 3], dtype='M8[D]')
+    data = Data(dates=dates)
+    assert isinstance(data.get_component('dates'), DateTimeComponent)
+
+
+def test_clone_meta():
+    # Regression test for a bug that caused metadata to not be preserved
+    # when saving/loading sessions.
+    class CustomObject(object):
+        pass
+    data1 = Data(x=[1, 2, 3])
+    data1.meta['a'] = 1
+    data1.meta['b'] = 'test'
+    data1.meta['c'] = CustomObject()
+    data2 = clone(data1)
+    assert data2.meta['a'] == 1
+    assert data2.meta['b'] == 'test'
+    assert 'c' not in data2.meta
+
+
+def test_update_coords():
+
+    # Make sure that when overriding coords, the world coordinate components
+    # are updated.
+
+    data1 = Data(x=[1, 2, 3])
+
+    assert len(data1.components) == 3
+
+    assert_equal(data1[data1.world_component_ids[0]], [0, 1, 2])
+
+    data2 = Data(x=[1, 2, 3])
+
+    assert len(data1.links) == 2
+    assert len(data2.links) == 2
+
+    data_collection = DataCollection([data1, data2])
+
+    assert len(data_collection.links) == 4
+
+    data_collection.add_link(LinkSame(data1.world_component_ids[0], data2.world_component_ids[0]))
+
+    assert len(data_collection.links) == 5
+
+    class CustomCoordinates(Coordinates):
+
+        def axis_label(self, axis):
+            return 'Custom {0}'.format(axis)
+
+        def world2pixel(self, *world):
+            return tuple([0.4 * w for w in world])
+
+        def pixel2world(self, *pixel):
+            return tuple([2.5 * p for p in pixel])
+
+    data1.coords = CustomCoordinates()
+
+    assert len(data1.components) == 3
+
+    assert_equal(data1[data1.world_component_ids[0]], [0, 2.5, 5])
+
+    assert sorted(cid.label for cid in data1.world_component_ids) == ['Custom 0']
+
+    # The link between the two world coordinates should be remove
+    assert len(data_collection.links) == 4
