@@ -1,13 +1,14 @@
 from __future__ import absolute_import, division, print_function
 
+import sys
 import numpy as np
 import pandas as pd
 from mock import MagicMock
 from pandas.util.testing import (assert_series_equal,
                                  assert_frame_equal)
 
-from ..component import (Component, DerivedComponent, CoordinateComponent,
-                         CategoricalComponent)
+from ...external.six import PY3
+from ..component import Component, DerivedComponent, CategoricalComponent
 from ..data import Data
 
 
@@ -40,13 +41,13 @@ class TestPandasConversion(object):
     def test_CoordinateComponent_conversion(self):
 
         d = Data(x=[1, 2, 3])
-        series = pd.Series(np.array([0, 1, 2], dtype=np.int))
+        series = pd.Series(np.array([0, 1, 2]))
         comp = d.get_component(d.get_pixel_component_id(0))
         assert_series_equal(series, comp.to_series())
 
     def test_Data_conversion(self):
 
-        d = Data(n=np.array([4, 5, 6, 7], dtype=np.int64))
+        d = Data(n=np.array([4, 5, 6, 7]))
         cat_comp = CategoricalComponent(np.array(['a', 'b', 'c', 'd']))
         d.add_component(cat_comp, 'c')
         link = MagicMock()
@@ -55,14 +56,19 @@ class TestPandasConversion(object):
         d.add_component(deriv_comp, 'd')
         order = [comp.label for comp in d.components]
 
-        frame = pd.DataFrame({
-            'n': [4, 5, 6, 7],
-            'c': ['a', 'b', 'c', 'd'],
-            'd': np.arange(4),
-            'Pixel Axis 0 [x]': np.arange(4),
-            'World 0': np.arange(4)
-        })
+        if sys.platform.startswith('win') and PY3:
+            world_0_dtype = np.int32
+        else:
+            world_0_dtype = np.int64
+
+        frame = pd.DataFrame()
+        frame['Pixel Axis 0 [x]'] = np.ogrid[0:4]
+        frame['World 0'] = np.arange(4).astype(world_0_dtype)
+        frame['n'] = np.array([4, 5, 6, 7])
+        frame['c'] = ['a', 'b', 'c', 'd']
+        frame['d'] = np.arange(4)
         out_frame = d.to_dataframe()
+
         assert_frame_equal(out_frame, frame)
         assert list(out_frame.columns) == order
 
