@@ -3,6 +3,7 @@ from __future__ import absolute_import, division, print_function
 import os
 
 from qtpy import QtCore, QtWidgets
+from qtpy.QtCore import Qt
 from glue.utils.qt import load_ui
 from glue.utils import nonpartial
 
@@ -21,6 +22,8 @@ class ComponentSelector(QtWidgets.QWidget):
        >>> widget = ComponentSelector()
        >>> widget.setup(data_collection)
     """
+
+    data_changed = QtCore.Signal()
     component_changed = QtCore.Signal()
 
     def __init__(self, parent=None):
@@ -41,6 +44,8 @@ class ComponentSelector(QtWidgets.QWidget):
         ds.currentIndexChanged.connect(nonpartial(self._set_components))
         self._ui.component_selector.currentItemChanged.connect(
             lambda *args: self.component_changed.emit())
+        self._ui.data_selector.currentIndexChanged.connect(
+            lambda *args: self.data_changed.emit())
 
     def set_current_row(self, row):
         """Select which component is selected
@@ -70,17 +75,40 @@ class ComponentSelector(QtWidgets.QWidget):
     def _set_components(self):
         """ Set list of component widgets to match current data set """
         index = self._ui.data_selector.currentIndex()
+        c_list = self._ui.component_selector
+        c_list.clear()
         if index < 0:
             return
         data = self._data[index]
-        cids = data.components
 
-        c_list = self._ui.component_selector
-        c_list.clear()
-        for c in cids:
-            item = QtWidgets.QListWidgetItem(c.label)
+        # Coordinate components
+        if len(data.coordinate_components) > 0:
+            item = QtWidgets.QListWidgetItem('Coordinate components')
+            item.setFlags(item.flags() & ~Qt.ItemIsEnabled)
             c_list.addItem(item)
-            c_list.set_data(item, c)
+            for c in data.coordinate_components:
+                item = QtWidgets.QListWidgetItem(c.label)
+                c_list.addItem(item)
+                c_list.set_data(item, c)
+
+        if len(set(data.primary_components) - set(data.coordinate_components)) > 0:
+            item = QtWidgets.QListWidgetItem('Main components')
+            item.setFlags(item.flags() & ~Qt.ItemIsEnabled)
+            c_list.addItem(item)
+            for c in data.primary_components:
+                if c not in data.coordinate_components:
+                    item = QtWidgets.QListWidgetItem(c.label)
+                    c_list.addItem(item)
+                    c_list.set_data(item, c)
+
+        if len(set(data.derived_components) & set(data.visible_components)) > 0:
+            item = QtWidgets.QListWidgetItem('Derived components')
+            item.setFlags(item.flags() & ~Qt.ItemIsEnabled)
+            c_list.addItem(item)
+            for c in data.derived_components:
+                item = QtWidgets.QListWidgetItem(c.label)
+                c_list.addItem(item)
+                c_list.set_data(item, c)
 
     def _set_data(self):
         """ Populate the data list with data sets in the collection """
@@ -116,6 +144,9 @@ class ComponentSelector(QtWidgets.QWidget):
 
     @data.setter
     def data(self, value):
+        if value is None:
+            self._ui.data_selector.setCurrentIndex(-1)
+            return
         for i, d in enumerate(self._data):
             if d is value:
                 self._ui.data_selector.setCurrentIndex(i)
@@ -149,5 +180,6 @@ def main():  # pragma: no cover
     w.show()
     app.exec_()
 
+
 if __name__ == "__main__":  # pragma: no cover
     main()
