@@ -27,8 +27,8 @@ ASTROPY_INSTALLED, requires_astropy = make_skipper('astropy',
                                                    label='Astropy')
 
 MATPLOTLIB_GE_14_INSTALLED, requires_matplotlib_ge_14 = make_skipper('matplotlib',
-                                                               label='Matplotlib >= 1.4',
-                                                               version='1.4')
+                                                                     label='Matplotlib >= 1.4',
+                                                                     version='1.4')
 
 ASTRODENDRO_INSTALLED, requires_astrodendro = make_skipper('astrodendro')
 
@@ -49,17 +49,16 @@ IPYTHON_INSTALLED, requires_ipython = make_skipper('IPython')
 requires_pil_or_skimage = pytest.mark.skipif(str(not SKIMAGE_INSTALLED and not PIL_INSTALLED),
                                              reason='Requires PIL or scikit-image')
 
-GINGA_INSTALLED, requires_ginga = make_skipper('ginga')
-
 PLOTLY_INSTALLED, requires_plotly = make_skipper('plotly')
 
 H5PY_INSTALLED, requires_h5py = make_skipper('h5py')
 
-PYQT4_INSTALLED, requires_pyqt4 = make_skipper('PyQt4')
 PYQT5_INSTALLED, requires_pyqt5 = make_skipper('PyQt5')
-PYSIDE_INSTALLED, requires_pyside = make_skipper('PySide')
+PYSIDE2_INSTALLED, requires_pyside2 = make_skipper('PySide2')
 
-QT_INSTALLED = PYQT4_INSTALLED or PYQT5_INSTALLED or PYSIDE_INSTALLED
+HYPOTHESIS_INSTALLED, requires_hypothesis = make_skipper('hypothesis')
+
+QT_INSTALLED = PYQT5_INSTALLED or PYSIDE2_INSTALLED
 
 SPECTRAL_CUBE_INSTALLED, requires_spectral_cube = make_skipper('spectral_cube',
                                                                label='spectral-cube')
