@@ -11,12 +11,12 @@ from glue.utils.qt.widget_properties import WidgetProperty
 
 from matplotlib import cm
 
-__all__ = ['mpl_to_qt4_color', 'qt4_to_mpl_color', 'cmap2pixmap',
+__all__ = ['mpl_to_qt_color', 'qt_to_mpl_color', 'cmap2pixmap',
            'tint_pixmap', 'QColorBox', 'ColorProperty', 'connect_color',
            'QColormapCombo']
 
 
-def mpl_to_qt4_color(color, alpha=None):
+def mpl_to_qt_color(color, alpha=None):
     """
     Convert a matplotlib color stirng into a Qt QColor object
 
@@ -42,7 +42,7 @@ def mpl_to_qt4_color(color, alpha=None):
     return QtGui.QColor(r * 255, g * 255, b * 255, a * 255)
 
 
-def qt4_to_mpl_color(qcolor):
+def qt_to_mpl_color(qcolor):
     """
     Convert a QColor object into a string that matplotlib understands
 
@@ -62,7 +62,7 @@ def qt4_to_mpl_color(qcolor):
     return str(hexid)
 
 
-def cmap2pixmap(cmap, steps=50, size=(100,100)):
+def cmap2pixmap(cmap, steps=50, size=(100, 100)):
     """
     Convert a maplotlib colormap into a QPixmap
 
@@ -84,7 +84,7 @@ def cmap2pixmap(cmap, steps=50, size=(100,100)):
     inds = np.linspace(0, 1, steps)
     rgbas = sm.to_rgba(inds)
     rgbas = [QtGui.QColor(int(r * 255), int(g * 255),
-                    int(b * 255), int(a * 255)).rgba() for r, g, b, a in rgbas]
+                          int(b * 255), int(a * 255)).rgba() for r, g, b, a in rgbas]
     im = QtGui.QImage(steps, 1, QtGui.QImage.Format_Indexed8)
     im.setColorTable(rgbas)
     for i in range(steps):
@@ -140,6 +140,11 @@ def connect_color(client, prop, widget):
 
     add_callback(client, prop, update_widget)
     widget.colorChanged.connect(nonpartial(update_prop))
+    update_widget(getattr(client, prop))
+
+
+from glue.external.echo.qt.autoconnect import HANDLERS
+HANDLERS['color'] = connect_color
 
 
 class QColorBox(QtWidgets.QLabel):
@@ -160,7 +165,7 @@ class QColorBox(QtWidgets.QLabel):
     def query_color(self):
         color = QtWidgets.QColorDialog.getColor(self._qcolor, parent=self)
         if color.isValid():
-            self.setColor(qt4_to_mpl_color(color))
+            self.setColor(qt_to_mpl_color(color))
 
     def setColor(self, color):
         self._color = color
@@ -170,7 +175,7 @@ class QColorBox(QtWidgets.QLabel):
         return self._color
 
     def on_color_change(self):
-        self._qcolor = mpl_to_qt4_color(self.color())
+        self._qcolor = mpl_to_qt_color(self.color())
         image = QtGui.QImage(70, 22, QtGui.QImage.Format_RGB32)
         try:
             image.fill(self._qcolor)
@@ -181,18 +186,21 @@ class QColorBox(QtWidgets.QLabel):
         self.setPixmap(pixmap)
 
 
+from glue.external.echo.qt.connect import UserDataWrapper
+
+
 class QColormapCombo(QtWidgets.QComboBox):
 
     def __init__(self, *args, **kwargs):
         super(QColormapCombo, self).__init__(*args, **kwargs)
         for label, cmap in config.colormaps:
-            self.addItem("", userData=cmap)
+            self.addItem("", userData=UserDataWrapper(cmap))
         self._update_icons()
 
     def _update_icons(self):
         self.setIconSize(QtCore.QSize(self.width(), 15))
         for index in range(self.count()):
-            cmap = self.itemData(index)
+            cmap = self.itemData(index).data
             icon = QtGui.QIcon(cmap2pixmap(cmap, size=(self.width(), 15), steps=200))
             self.setItemIcon(index, icon)
 
@@ -208,7 +216,7 @@ if __name__ == "__main__":
     app = get_qapp()
 
     label = QColorBox()
-    label.resize(100,100)
+    label.resize(100, 100)
     label.show()
     label.raise_()
     app.exec_()
