@@ -1,6 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
 import os
+import sys
 from contextlib import contextmanager
 
 from qtpy import QtCore, QtWidgets
@@ -8,10 +9,11 @@ from qtpy.QtCore import Qt
 from qtpy.uic import loadUi
 from glue.utils.qt import get_text
 
-__all__ = ['update_combobox', 'GlueTabBar', 'load_ui', 'process_dialog']
+__all__ = ['update_combobox', 'GlueTabBar', 'load_ui', 'process_dialog',
+           'combo_as_string', 'qurl_to_path']
 
 
-def update_combobox(combo, labeldata):
+def update_combobox(combo, labeldata, default_index=0):
     """
     Redefine the items in a QComboBox
 
@@ -48,19 +50,26 @@ def update_combobox(combo, labeldata):
         current = None
 
     combo.clear()
-    index = 0
+    index = None
     for i, (label, data) in enumerate(labeldata):
         combo.addItem(label, userData=data)
-        if data is current:
+        if data is current or data == current:
             index = i
-    combo.blockSignals(False)
+
+    if default_index < 0:
+        default_index = combo.count() + default_index
+
+    if index is None:
+        index = min(default_index, combo.count() - 1)
     combo.setCurrentIndex(index)
 
+    combo.blockSignals(False)
+
     # We need to force emit this, otherwise if the index happens to be the
     # same as before, even if the data is different, callbacks won't be
-    # called.
-    if idx == index or idx == -1:
-        combo.currentIndexChanged.emit(index)
+    # called. So we block the signals until just before now then always call
+    # callback manually.
+    combo.currentIndexChanged.emit(index)
 
 
 class GlueTabBar(QtWidgets.QTabBar):
@@ -68,14 +77,32 @@ class GlueTabBar(QtWidgets.QTabBar):
     def __init__(self, *args, **kwargs):
         super(GlueTabBar, self).__init__(*args, **kwargs)
 
-    def rename_tab(self, index=None):
-        """ Prompt user to rename a tab
-        :param index: integer. Index of tab to edit. Defaults to current index
+    def choose_rename_tab(self, index=None):
+        """
+        Prompt user to rename a tab
+
+        Parameters
+        ----------
+        index : int
+            Index of tab to edit. Defaults to current index
         """
         index = index or self.currentIndex()
         label = get_text("New Tab Label")
         if not label:
             return
+        self.rename_tab(index, label)
+
+    def rename_tab(self, index, label):
+        """
+        Updates the name used for given tab
+
+        Parameters
+        ----------
+        index : int
+            Index of tab to edit. Defaults to current index
+        label : str
+            New label to use for this tab
+        """
         self.setTabText(index, label)
 
     def mouseDoubleClickEvent(self, event):
@@ -83,7 +110,7 @@ class GlueTabBar(QtWidgets.QTabBar):
             return
         index = self.tabAt(event.pos())
         if index >= 0:
-            self.rename_tab(index)
+            self.choose_rename_tab(index)
 
 
 def load_ui(path, parent=None, directory=None):
@@ -162,7 +189,9 @@ def process_dialog(delay=0, accept=False, reject=False, function=None):
     def wrapper():
         from glue.utils.qt import get_qapp
         app = get_qapp()
-        dialog = app.focusWidget().window()
+        # Make sure that any window/dialog that needs to be shown is shown
+        app.processEvents()
+        dialog = app.activeWindow()
         function(dialog)
 
     timer = QtCore.QTimer()
@@ -172,3 +201,29 @@ def process_dialog(delay=0, accept=False, reject=False, function=None):
     timer.start()
 
     yield
+
+
+def combo_as_string(combo):
+    """
+    Return the text labels of a combo box as a string to make it easier to
+    check the content of a combo box in tests.
+    """
+    items = [combo.itemText(i) for i in range(combo.count())]
+    return ":".join(items)
+
+
+def qurl_to_path(url):
+    """
+    Convert a local QUrl to a normal path
+    """
+
+    # Get path to file
+    path = url.path()
+
+    # Workaround for a Qt bug that causes paths to start with a /
+    # on Windows: https://bugreports.qt.io/browse/QTBUG-46417
+    if sys.platform.startswith('win'):
+        if path.startswith('/') and path[2] == ':':
+            path = path[1:]
+
+    return path
