@@ -5,19 +5,17 @@ import pytest
 from ..misc import as_variable_name, file_format, DeferredMethod, nonpartial, lookup_class, as_list
 
 
-def test_as_variable_name():
-    def check(input, expected):
-        assert as_variable_name(input) == expected
+INPUT_EXPECTED = [('x', 'x'),
+                  ('x2', 'x2'),
+                  ('2x', '_2x'),
+                  ('x!', 'x_'),
+                  ('x y z', 'x_y_z'),
+                  ('_XY', '_XY')]
 
-    tests = [('x', 'x'),
-             ('x2', 'x2'),
-             ('2x', '_2x'),
-             ('x!', 'x_'),
-             ('x y z', 'x_y_z'),
-             ('_XY', '_XY')
-             ]
-    for input, expected in tests:
-        yield check, input, expected
+
+@pytest.mark.parametrize(('input', 'expected'), INPUT_EXPECTED)
+def test_as_variable_name(input, expected):
+    assert as_variable_name(input) == expected
 
 
 class TestFileFormat(object):
@@ -77,17 +75,23 @@ def test_nonpartial():
 
 def test_lookup_class():
 
-    lookup_class('glue.utils.misc.DeferredMethod') is DeferredMethod
+    assert lookup_class('glue.utils.misc.DeferredMethod') is DeferredMethod
 
     with pytest.raises(ValueError) as exc:
-        lookup_class('gluh.utils.misc.DeferredMethod') is None
+        assert lookup_class('gluh.utils.misc.DeferredMethod') is None
     assert exc.value.args[0] == "Module 'gluh.utils.misc' not found"
 
     with pytest.raises(ValueError) as exc:
-        lookup_class('glue.utils.misc.DeferredMethods') is None
+        assert lookup_class('glue.utils.misc.DeferredMethods') is None
     assert exc.value.args[0] == "Object 'glue.utils.misc.DeferredMethods' not found"
 
 
+def test_lookup_class_builtins():
+
+    assert lookup_class('__builtin__.dict') is dict
+    assert lookup_class('builtins.dict') is dict
+
+
 def test_as_list():
     as_list(1) == [1]
     as_list([2, 3]) == [2, 3]
